package java2gomd;
import java.lang.*;
import java.util.*;
import java.io.*;

/**
* Represents a whole cluster with Gomd running on each node.
* Each instance has its own hash, usable to check if we have multiple connections to the same cluster.
* @version 1.0
* @author roeles
* @see java.util.Hashtable
*/
public class Cluster extends GomdObject
{
	/**
	* Creates a Cluster object communicating via GomdClient c
	* @param c The GomdClient it is going to use to get info from. This GomdClient is also used by all other objects in this Cluster
	*/
	public Cluster(GomdClient c)
	{
		super(c);
	}

	/**
	* Returns a hash unique for every cluster.
	* This helps us checking if we have multiple connections to the same cluster.
	* @return int representing the hashCode.
	*/
	public int getHash() throws IOException
	{
		String summary = "" + getSpeed() + getMemory();
		return summary.hashCode();
	}

	/**
	* Returns the average load of all the nodes in the cluster in percents.
	* It requests all nodes and then requests the load of each node.
	* @return int representing the avarage load in the cluster
	*/
	public int getLoad() throws IOException
	{
		Enumeration e = (getNodes()).elements();
		int load = 0;
		int i;
		for(i=0;e.hasMoreElements();i++)
		{
			Node n = (Node)e.nextElement();
			load = load + n.load();
		}
		return load/i;
	}

/*	public int getClusterCpuNumber() // Broken due to one cpu return from gomd
	{
	Enumeration e = (getNodes()).elements();
	int cpus;
	while(e.hasMoreElements())
		cpus =

	//Get all nodes
	//Get number of cpus of every node
	//Calculate
	}
*/


	/**
	* Returns the total amount of memory psysically available in the cluster
	* It requests all nodes and then requests the memory available in each node.
	* @return long representing the total psysical memory in the cluster
	*/
	public long getMemory() throws IOException
	{
		long memory = 0;
		Enumeration e = (getNodes()).elements();
		while(e.hasMoreElements())
		{
			Node n = (Node)e.nextElement();
			memory = memory + n.tmem();
		}
		return memory;
	}

	/**
	* Returns the total speed of all nodes in the cluster running gomd.
	* It requests all nodes and then requests the speed of each node.
	* @return long respresenting the total speed of all nodes in the cluster.
	*/
	public long getSpeed() throws IOException
	{
		long speed = 0;
		Enumeration e = (getNodes()).elements();
		while(e.hasMoreElements())
		{
			Node n = (Node)e.nextElement();
			speed = speed + n.speed();
		}
		return speed;
	}

	/**
	* Returns the Loadbalancing efficiency of the cluster
	* It calculates 100%-(maxload - minload).
	* This function is equivalent to the one found in openMosixView
	* @return int respresenting the percentage of loadbalancing performed by the cluster.
	*/
	public int getLoadbalancingEfficiency() throws IOException
	{
		int efficiency;
		int maxload=0, minload=100, load=0;
		Enumeration e = (getNodes()).elements();
		while(e.hasMoreElements())
		{
			load = ((Node)e.nextElement()).load();
			if(maxload < load)
				maxload = load;
			if(minload > load)
				minload = load;
		}
		return (100-(maxload - minload));

	}

	/**
	* Returns a Hashtable containing all available nodes in the cluster.
	* <BR><code> Node n = (Node)getNodes().get("257"); </code>
	* @return Hashtable filled with all nodes.
	*/
	public Hashtable getNodes() throws IOException
	{
		clientConnection.send("get nodelist");
		String nodes[] = clientConnection.grep("AVAILABLE_NODE_LIST");

		Hashtable nodeHash = new Hashtable();
		for(int i=0; i<nodes.length; i++)
			nodeHash.put(nodes[i],new Node(clientConnection, nodes[i]));
		return nodeHash;
	}

	/**
	* Runs a command on every node using Secure Cluster eXecution
	* @return int[] representing the exit status of the program ran on each node
	*/
	public int[] cwc(String cmd) throws IOException
	{
		int bla[] = {1,2};
		return bla;
	}

}
