package java2gomd;
import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.net.*;

/**
* The connection to the Gomd daemon
* @version 1.0
* @author roeles
* @see java.util.Hashtable
*/
public class GomdClient
{
	private Socket clientSocket;
	private BufferedReader input;
	private PrintStream output;
	private String hostname;
	private int port;


	private String lastRequest = new String("");

	private long bytesSent;
	private long bytesReceived;

	/**
	* Creates a GomdClient, the connection to the Gomd daemon
	* @param gomd_hostname The hostname or IPv4 address to connect to
	* @param gomd_port The port on wich Gomd is listening
	*/
	public GomdClient(String gomd_hostname, int gomd_port) throws IOException
	{
			System.out.println("GomdClient::GomdClient(): New Client Made");
		setAddress(gomd_hostname, gomd_port);
		connect();
	}


	/**
	* Sets the address and port of the Gomd to connect to.
	* @param hostname The hostname or IPv4-address to connect to
	* @param port The port on wich Gomd is listening
	*/
	public void setAddress(String hostname, int port)
	{
		this.hostname = hostname;
			System.out.println("GomdClient::setAddress(): Hostname set to " + this.hostname);
		this.port = port;
			System.out.println("GomdClient::setAddress(): Port set to " + this.port);
	}

	/**
	* Connects to the remote Gomd and attaches the input and output streams
	* @return boolean respresenting failure(1) or success(0) of connecting.
	*/
	public boolean connect() throws UnknownHostException, IOException
	{
		clientSocket = new Socket(hostname, port);
			System.out.println("GomdClient::connect(): connection made");
		input = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
			System.out.println("GomdClient::connect(): DataInputStream connected");
		output = new PrintStream(clientSocket.getOutputStream());
			System.out.println("GomdClient::connect(): PrintStream connected");

		return true;
	}

	/**
	* Called by java's garbage collector when removed
	* Calls close() to close the connection neatly.
	*/
	protected void finalize() throws IOException
	{
			System.out.println("GomdClient::finalize(): Closing");
		close();
			System.out.println("GomdClient::finalize(): called close()");
	}


	/**
	* Detaches the input and output streams and then closes the connection.
	*/
	public void close() throws IOException
	{
		output.close();
			System.out.println("GomdClient::close(): Printstream closed");
		input.close();
			System.out.println("GomdClient::close(): DataInputStream closed");
		clientSocket.close();
			System.out.println("GomdClient::close(): connection closed");
	}


	//NEW BLOCK OF CODE
	public String[] request(String data) throws IOException
	{
		send(data);
		return read();
	}

	public String[] read() throws IOException
	{
		return read(-1);
	}


	public String[] read(int num) throws IOException
	{
		String[] reply = {"nothing", "returned"};
		for(int i=0; (input.ready() && i!=num); i++)
		{
				System.out.println("GomdClient::read(): more data is available");
			String readLine = input.readLine();
			bytesReceived = bytesReceived + (readLine.getBytes()).length;
				System.out.println("GOMD: " + readLine);
			reply[i] = readLine;
		}
		return reply;
	}

/*  Multiline functions are WIP, will be enabled ASAP
	//
	// Filters the requested values from the given <B>multiline</B> data
	// @param pattern The value's to return
	// @param data The data to filter
	// @return String[][] with the requested values. This is indexed like <code>String[filtervalue][occurence]</code>
	//
	public static String[][] filter(String[] pattern, String[] data)
	{
		String[][] result;
		for(int filtervalue=0; filtervalue < pattern.length; filtervalue++)

		return result;
	}


	//
	// Filters the requested value from the given data
	// @param pattern The value to return
	// @param data The data to filter
	// @return String[] containing all occurences of pattern in data
	//
	public static String[] filter(String pattern, String data[])
	{
	}
*/




	/**
	* Filters the requested values from the given data
	* @param pattern The value's to return
	* @param data The data to filter
	* @return String[][] with the requested values This is indexed like <code>String[filtervalue][occurence]</code>
	*/
	public static String[][] filter(String[] pattern, String data)
	{
		LinkedList ll = new LinkedList();
		for(int i=0; i< pattern.length; i++)
			ll.add(filter(pattern[i], data));
		return (String[][])ll.toArray();
		//String[][] result;
		//for(int i=0;i < pattern.length; i++)
		//	result[i] = filter(pattern[i], data);
		//return result;
	}

	/**
	* Filters the requested value from the given data
	* @param pattern The value to return
	* @param data The data to filter
	* @return String[] with the requested value
	*/
	public static String[] filter(String pattern, String data)
	{
		String[] values = {"nothing", "returned"};
		for(int i=0; i < values.length; i++)
				{
					if(data.indexOf(pattern) != -1)
					{
							System.out.println("GomdClient::grep(): " + data + " contains " + pattern + " at " + data.indexOf(pattern));
						String tmp = data.substring(data.indexOf(pattern));
						String tmp2 = tmp.substring(tmp.indexOf(":")+1,tmp.indexOf(")"));
						values = tmp2.split(",");

						break;

					}
				}
		return values;

	}
	//END OF NEW BLOCK


	/**
	* Sends a request and returns the specified data
	* @return String[] representing the value's Gomd returned
	* @param data The query to send
	* @param pattern The value to return
	*/
	public String[] query(String data, String pattern) throws IOException
	{
		System.out.println("GomdClient::query");
		send(data);
		return grep(pattern);
	}


	/**
	* Sends a user-specified string
	* @param data The String to send to Gomd
	*/
	public void send(String data)
	{
		try { Thread.sleep(10); } catch(Exception e) { }
		lastRequest = data;
		output.println(data);
		bytesSent = bytesSent + (data.getBytes()).length;
			System.out.println("GomdClient::send(): Sent " + data + " " + bytesSent + " Bytes total sent");
	}

	/**
	* Filters the user-specified value from Gomd's reply
	* @param pattern The value to return
	* @return String[] listing all the value's returned by Gomd
	* Example: <BR>Returns {1,2,3,4,5,6} when called like <BR>
	* <code>GomdClient.send("get pidlist 257");<BR>GomdClient.grep("PIDS_LIST");</code><BR>
	* and Gomd Returns ($GARBAGE)(PIDS_LIST:1,2,3,4,5,6)($GARBAGE)
	*/
	public String[] grep(String pattern) throws IOException
	{
		String grepped[] = {"nothing", "returned"}; //In case or no answer
			System.out.println("GomdClient::grep(): Filtering for " + pattern);
		while(input.ready())
		{
				System.out.println("GomdClient::grep(): more data is available");
			String readLine = input.readLine();
			bytesReceived = bytesReceived + (readLine.getBytes()).length;
				System.out.println("GOMD: " + readLine);

			if(readLine.indexOf(pattern) != -1)
			{
					System.out.println("GomdClient::grep(): " + readLine + " contains " + pattern + " at " + readLine.indexOf(pattern));
				String tmp = readLine.substring(readLine.indexOf(pattern));
				String tmp2 = tmp.substring(tmp.indexOf(":")+1,tmp.indexOf(")"));
				grepped = tmp2.split(",");

				break;

			}
		}
		return grepped;
	}


	/**
	* Returns the number of bytes received by this connection
	* @return long representing the number of bytes received by this connection.
	*/
	public long getBytesReceived()
	{
		return bytesReceived;
	}

	/**
	* Returns the number of bytes sent by this connection
	* @return long representing the number of bytes sent by this connection.
	*/
	public long getBytesSent()
	{
		return bytesSent;
	}

}
