package java2gomd;
import java.io.*;
import java.util.regex.*;
import java.net.*;

/**
* The connection to the Gomd daemon
* @version 1.0
* @author roeles
* @see java.util.Hashtable
*/
public class GomdClient
{
	private Socket clientSocket;
	private BufferedReader input;
	private PrintStream output;
	private String hostname;
	private int port;

	private long bytesSent;
	private long bytesReceived;

	/**
	* Creates a GomdClient, the connection to the Gomd daemon
	* @param gomd_hostname The hostname or IPv4 address to connect to
	* @param gomd_port The port on wich Gomd is listening
	*/
	public GomdClient(String gomd_hostname, int gomd_port) throws IOException
	{
			System.out.println("GomdClient::GomdClient(): New Client Made");
		setAddress(gomd_hostname, gomd_port);
		connect();
	}


	/**
	* Sets the address and port of the Gomd to connect to.
	* @param hostname The hostname or IPv4-address to connect to
	* @param port The port on wich Gomd is listening
	*/
	public void setAddress(String hostname, int port)
	{
		this.hostname = hostname;
			System.out.println("GomdClient::setAddress(): Hostname set to " + this.hostname);
		this.port = port;
			System.out.println("GomdClient::setAddress(): Port set to " + this.port);
	}

	/**
	* Connects to the remote Gomd and attaches the input and output streams
	* @return boolean respresenting failure(1) or success(0) of connecting.
	*/
	public boolean connect() throws UnknownHostException, IOException
	{
		clientSocket = new Socket(hostname, port);
			System.out.println("GomdClient::connect(): connection made");
		input = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
			System.out.println("GomdClient::connect(): DataInputStream connected");
		output = new PrintStream(clientSocket.getOutputStream());
			System.out.println("GomdClient::connect(): PrintStream connected");

		return true;
	}

	/**
	* Called by java's garbage collector when removed
	* Calls close() to close the connection neatly.
	*/
	protected void finalize() throws IOException
	{
			System.out.println("GomdClient::finalize(): Closing");
		close();
			System.out.println("GomdClient::finalize(): called close()");
	}


	/**
	* Detaches the input and output streams and then closes the connection.
	*/
	public void close() throws IOException
	{
		output.close();
			System.out.println("GomdClient::close(): Printstream closed");
		input.close();
			System.out.println("GomdClient::close(): DataInputStream closed");
		clientSocket.close();
			System.out.println("GomdClient::close(): connection closed");
	}

	/**
	* Sends a user-specified string
	* @param data The String to send to Gomd
	*/
	public void send(String data)
	{
		try { Thread.sleep(10); } catch(Exception e) { }
		output.println(data);
		bytesSent = bytesSent + (data.getBytes()).length;
			System.out.println("GomdClient::send(): Sent " + data + " " + bytesSent + " Bytes total sent");
	}

	/**
	* Filters the user-specified value from Gomd's reply
	* @param pattern The value to return
	* @return String[] listing all the value's returned by Gomd
	* Example: <BR>Returns {1,2,3,4,5,6} when called like <BR>
	* <code>GomdClient.send("get pidlist 257");<BR>GomdClient.grep("PIDS_LIST");</code><BR>
	* and Gomd Returns ($GARBAGE)(PIDS_LIST:1,2,3,4,5,6)($GARBAGE)
	*/
	public String[] grep(String pattern) throws IOException
	{
		String grepped[] = {"nothing", "returned"}; //In case or no answer
			System.out.println("GomdClient::grep(): Filtering for " + pattern);
		while(input.ready())
		{
				System.out.println("GomdClient::grep(): more data is available");
			String readLine = input.readLine();
			bytesReceived = bytesReceived + (readLine.getBytes()).length;
				System.out.println("GOMD: " + readLine);

			if(readLine.indexOf(pattern) != -1)
			{
					System.out.println("GomdClient::grep(): " + readLine + " contains " + pattern + " at " + readLine.indexOf(pattern));
				String tmp = readLine.substring(readLine.indexOf(pattern));
				String tmp2 = tmp.substring(tmp.indexOf(":")+1,tmp.indexOf(")"));
				grepped = tmp2.split(",");

				break;

			}
		}
		return grepped;
	}


	/**
	* Returns the number of bytes received by this connection
	* @return long representing the number of bytes received by this connection.
	*/
	public long getBytesReceived()
	{
		return bytesReceived;
	}

	/**
	* Returns the number of bytes sent by this connection
	* @return long representing the number of bytes sent by this connection.
	*/
	public long getBytesSent()
	{
		return bytesSent;
	}

}
