package java2gomd;
import java.lang.*;
import java.util.*;
import java.io.*;

/**
* Represents a node (a computer in the cluster)
* @version 1.0
* @author roeles
*/
public class Node extends GomdObject
{
	private int id;

	/**
	* Creates a new node wich will retreive info over client with openMosix id node
	* @param client The GomdClient it will use to retreive it's info.
	* @param node It's openMosix id in the cluster.
	*/
	public Node(GomdClient client, String node)
	{
		super(client);
		this.id = Integer.parseInt(String.valueOf(node));
	}

//	public Hashtable cpus()
//	{
//
//	}

	/**
	* Returnes the load of this node
	* @return int representing the load in percent.
	*/
	public int load() throws IOException
	{
		clientConnection.send("get load " + id);
		String temp[] = clientConnection.grep("LOAD_OF_NODE_#" + id);
		return Integer.parseInt(String.valueOf(temp[0]));
	}

//	public Boolean dead()
//	{
//	//"get nodelist"
//	//find own node in list->dead
//	}

	/**
	* Returnes the status of this node
	* @return int representing the status of this node as shown in /proc/hpc/$node/status
	*/
	public int status() throws IOException
	{
		clientConnection.send("get status " + id);
		String temp[] = clientConnection.grep("STATUS_OF_NODE_#" + id);
		return Integer.parseInt(String.valueOf(temp[0]));
	}

	/**
	* Returnes the total memory in this node
	* @return long representing the node's memory in bytes
	*/
	public long tmem() throws IOException
	{
		clientConnection.send("get tmem " + id);
		String temp[] = clientConnection.grep("TMEM_OF_NODE_#" + id);
		return Long.parseLong(String.valueOf(temp[0]));
	}

	/**
	* Returnes the speed of this node
	* @return long representing the speed of this node as specified by openMosix in /proc/hpc/$node/speed
	*/
	public long speed() throws IOException
	{
		clientConnection.send("get speed " + id);
		String temp[] = clientConnection.grep("SPEED_OF_NODE_#" + id);
		return Integer.parseInt(String.valueOf(temp[0]));
	}

	/**
	* Returnes a list of processes
	* @return Hashtable with all the processes running on this node
	*/
	public Hashtable pidlist() throws IOException
	{
		clientConnection.send("get pidlist " + id);
		String temp[] = clientConnection.grep("PIDS_LIST");
		Hashtable pidlist = new Hashtable();
		for(int i=0;i<temp.length;i++)
			pidlist.put(temp[i],new Process(clientConnection, temp[i]));
		return pidlist;
	}

//	public Hashtable mountlist()
//	{
//	}

	/**
	* Returnes the uptime of this node
	* @return double representing the uptime of this node in seconds.
	*/
	public double uptime() throws IOException
	{
		clientConnection.send("get uptime " + id);
		String temp[] = clientConnection.grep("UPTIME");
		return Double.parseDouble(String.valueOf(temp[0]));
	}

	/**
	* Returnes the idletime of this node
	* @return double representing the idletime of this node in seconds.
	*/
	public double idletime() throws IOException
	{
		clientConnection.send("get uptime " + id);
		String temp[] = clientConnection.grep("IDLETIME");
		return Double.parseDouble(String.valueOf(temp[0]));
	}

	/**
	* Returnes all network interfaces on this node
	* @return Hashtable with alle the network interfaces on this node
	*/
	public Hashtable nics() throws IOException
	{
	//try all interfaces: lo, eth0, eth1, eth2, eth3
	//if netload != 0x00000000 -> maak new nicje
		Hashtable h = new Hashtable();
		for(int i=0; i<4;i++)
		{
			clientConnection.send("get netload eth" + i);
			String temp[] = clientConnection.grep("NETWORK_LOAD");
			if(temp[0] != "0x00000000" && temp[0] != "nothing")
				h.put("eth" + i,new Nic(clientConnection, "eth" + i));
		}
		return h;
	} //eth0, eth1

	/**
	* Executes a specified command on this node
	* @return int representing the exit status of the command
	*/
	public int cmd(String command)
	{
		clientConnection.send("cmd " + command);
		return -1;
	}
}
